        program cy2008
c
c This code computes the population mean and (total) standard
c deviation of ln(PSA) using the Chiou and Youngs 2008 NGA model.
c
c This code has been sucessfully compilied by Lahey/Fujitsu Fortran 95
c Compiler Release 5.60h
c
c  User Guide
c  Program prompts for input file and output file
c
c  Format for input file
c
c  First record (a)
c  Line     C   Header record
c
c
c  Second record (*)
c  PERIOD       Spectral period (sec), enter -2 for peak velocity
c  M            Magnitude
c  F_RV         Enter 1 for reverse faulting, 0 otherwise default is SS)
c  F_NM         Enter 1 for normal faulting, 0 otherwise (default is SS)
c  Z_TOR        Depth to top of rupture (km)
c  DELTA        Fault dip (degrees)
c  V_S30        Average shear wave velocity (m/s)
c  F_Inferred   Enter 1 if Vs30 is inferred, 0 if measured
c  Z1           Depth to Vs = 1 km/sec (m), ener -999 for default
c  R_Rup        Distance to rupture plane (km)
c  R_JB         Joyner-Boore distance (km)
c  R_x          Normal distance to strike of fault (km), negative on footwall
c
c  Enter -999 for period and 0 for all other parameters to stop
c
c.......NOTES:
c
c  1. If input parameter Z1 is given as -999, then the default Z1 value (Equation 1
c     in Chiou and Youngs, 2008) is used for the computation.
c  2. Factor F_HW is not part of the input; it'll be set by the program
c     based on the sign of paramter R_x.
c  3. F_Inferred = 1 if Vs30 is inferred
c                = 0 if Vs30 is measured
c  4. Total stdev is evaluated using the approximate method recommended in
c     CY2008 (Equation 21 of Chiou and Youngs, 2008).
c  5. This code is for mainshock only. Aftershock terms are not implemented.
c  6. For prediction of PGV, use PERIOD = -2.
c

        implicit none

        include 'CY2008.h'

        real psa, psa_ref, NL0, tau, sigma_NL0, total_app, pi, d2r
        integer iprd, i
        character*80 line, inputFile, outputFile

         pi = atan(1.0)*4.0
         d2r = pi/180.0

1000     write (*,*) 'Give Input File Name (q to quit): '
         read (*,'(a)') inputFile
         if (inputFile.eq.'q') go to 9999
         open (2,file=inputFile)

         write (*,*) 'Give Output File Name: '
         read (*,'(a)') outputFile
         open (9,file=outputFile)

         write (9,'(a)') 'Chiou and Youngs, 2008'
         write (9,*)
     1 'PERIOD    M   R_Rup    R_JB     R_x  V_S30 F_VS     Z1 F_RV ',
     1 'F_NM Z_TOR   DIP PSA_Ref     PSA     NL0     Tau Sigma_N0 ',
     1 'Total_Sigma'

c
c Read predictor variables
c

        read (2,'(a)') line
10      read (2,*,end=999) PERIOD, M, F_RV, F_NM, Z_TOR, DELTA,
     1             V_S30, F_Inferred, Z1, R_Rup, R_JB, R_x
        if (PERIOD.lt.-2.or.PERIOD.GT.10.0) go to 999

        do i=1,nprd
                if (PERIOD.eq.prd(i)) go to 11
        enddo
        write (*,'(1x,a2,f6.3, a27)')
     1         '* ', Period, '-sec period is not in CY2008 model.'
        stop

11      continue
        iprd = i

        F_Measured = 0.0
        if (F_Inferred.eq.0) F_Measured = 1.0

        if (F_RV+F_NM.gt.1) then
             write (*,*) 'F_RV and F_NM can not be 1 at the same time'
             stop
        endif
        if (R_JB.gt.R_Rup) then
             write (*,*) 'R_JB can not be greater than R_Rup'
             stop
        endif
        if (R_Rup.lt.Z_TOR) then
             write (*,*) 'R_Rup can not be less than Z_TOR'
             stop
        endif

        if (Z1.le.-999) then
             Z1 = exp(28.5-3.82/8*log(V_S30**8+378.8**8))
        endif

c
c Compute population mean and total stdev
c
        call CY2008_NGA(iprd, M, R_Rup, R_JB, R_x, V_S30, F_Measured,
     1             F_Inferred, Z1, DELTA, Z_TOR, F_RV, F_NM,
     1             psa_ref, psa, NL0, tau, sigma_NL0, total_app)

        write (9,1) PERIOD, M, R_Rup, R_JB, R_x, V_S30, F_Inferred, Z1,
     1              F_RV, F_NM, Z_TOR, DELTA,
     1              psa_ref, psa, NL0, tau, sigma_NL0, total_app

1       format (f7.3,f5.2,3f8.2,f7.1,f5.1,f7.1,3f5.1,f7.2,6f8.4)
        go to 10

999     continue
        go to 1000

9999    continue
        stop

        end program cy2008

c-----------------------------------------------------------------------
        subroutine CY2008_NGA(iprd, M, R_Rup, R_JB, R_x, V_S30,
     1                   F_Measured, F_Inferred, Z1, DELTA, Z_TOR,
     1                   F_RV, F_NM,
     1                   psa_ref, psa, NL0, tau, sigma_NL0, total_app)

        implicit none
        include 'CY2008.h'

        real cc, gamma, cosDELTA, psa, psa_ref, NL0, tau, sigma_NL0,
     1       total_app
        real pi, d2r, r1, r2, r3, r4, fw, hw, a, b, c, rkdepth
        integer iprd, sa


        pi = atan(1.0)*4.0
        d2r = pi/180.0

        cc = c5(iprd)* cosh(c6(iprd) * max(M-cHM(iprd),0))
        gamma = cgamma1(iprd) +
     1          cgamma2(iprd)/cosh(max(M-cgamma3(iprd),0))
        cosDELTA = cos(DELTA*d2r)

c Magnitude scaling
        r1 = c1(iprd) + c2(iprd) * (M-6.0) +
     1       (c2(iprd)-c3(iprd))/cn(iprd) *
     1             log(1.0 + exp(cn(iprd)*(cM(iprd)-M)))

c Near-field magnitude and distance scaling
        r2 = c4(iprd) * log(R_Rup + cc)

c Far-field distance scaling
        r3 = (c4a(iprd)-c4(iprd)) *
     1            log(sqrt(R_Rup*R_Rup+cRB(iprd)*cRB(iprd))) +
     1       R_Rup * gamma

c Scaling with other source variables (F_RV, F_NM, and Z_TOR)
        r4 = c1a(iprd)*F_RV +
     1       c1b(iprd)*F_NM +
     1       c7(iprd)*(Z_TOR - 4)

c HW effect
        if (R_x .lt. 0) then
          hw = 0.0
        else
          hw = c9(iprd) * tanh(R_x*cosDELTA**2/c9a(iprd)) *
     1        (1.0 - sqrt(R_JB**2+Z_TOR**2)/(R_Rup + 0.001))
        endif




        psa_ref = r1+r2+r3+r4+hw

c Soil effect: linear response
        a = phi1(iprd) * min(log(V_S30/1130), 0)

c Soil effect: nonlinear response
        b = phi2(iprd) *
     1(exp(phi3(iprd)*(min(V_S30,1130)-360))-exp(phi3(iprd)*(1130-360)))

        c = phi4(iprd)

c Corrections to ln(Vs30) scaling
c
c !!  NOTE: On the 3rd line, I cap max(0,Z1-15) at 300 to avoid
c !!          the numerical overflow of intrinsic function 'cosh'.
c
        rkdepth = phi5(iprd) *
     1        ( 1 - 1.0/cosh(phi6(iprd)*max(0,Z1-phi7(iprd)))) +
     1        phi8(iprd)/cosh(0.15*min(max(0, Z1-15),300))


c....... Polulation mean of ln(psa) (eta=0)

        psa = exp(psa_ref + (a + b * log((exp(psa_ref)+c)/c)) + rkdepth)
        psa_ref = exp(psa_ref)


c....... Total variance of ln(psa) about the population mean:
c          The approximate method (Equation 21)

        NL0 = b * psa_ref/(psa_ref+c)

        tau = tau1(iprd) +
     1            (tau2(iprd)-tau1(iprd))/2*(min(max(M,5),7)-5)

        sigma_NL0 = sigma1(iprd) +
     1              (sigma2(iprd)-sigma1(iprd))/2*(min(max(M,5),7)-5)

        sigma_NL0 = sigma_NL0 *
     1        sqrt(0.7*F_Measured+F_Inferred*sigma3(iprd)+(1+NL0)**2)

        total_app = sqrt((tau*(1+NL0))**2+sigma_NL0**2)

        return

        end subroutine CY2008_NGA


        block data CY2008Coefficients

        include 'cy2008.h'

      data prd     / -1,     -2,      0.0100, 0.0200, 0.0300,
     1                0.0400, 0.0500, 0.0750, 0.1000, 0.1500,
     1                0.2000, 0.2500, 0.3000, 0.4000, 0.5000,
     1                0.7500, 1.0000, 1.5000, 2.0000, 3.0000,
     1                4.0000, 5.0000, 7.5000,10.0000/


      data c1      / -1.2687, 2.2884,-1.2687,-1.2515,-1.1744,
     1               -1.0671,-0.9464,-0.7051,-0.5747,-0.5309,
     1               -0.6352,-0.7766,-0.9278,-1.2176,-1.4695,
     1               -1.9278,-2.2453,-2.7307,-3.1413,-3.7413,
     1               -4.1814,-4.5187,-5.1224,-5.5872/

      data c1a     /  0.1000, 0.1094, 0.1000, 0.1000, 0.1000,
     1                0.1000, 0.1000, 0.1000, 0.1000, 0.1000,
     1                0.1000, 0.1000, 0.0999, 0.0997, 0.0991,
     1                0.0936, 0.0766, 0.0022,-0.0591,-0.0931,
     1               -0.0982,-0.0994,-0.0999,-0.1000/

      data c1b     / -0.2550,-0.0626,-0.2550,-0.2550,-0.2550,
     1               -0.2550,-0.2550,-0.2540,-0.2530,-0.2500,
     1               -0.2449,-0.2382,-0.2313,-0.2146,-0.1972,
     1               -0.1620,-0.1400,-0.1184,-0.1100,-0.1040,
     1               -0.1020,-0.1010,-0.1010,-0.1000/

      data c2 /      1.06,1.06,1.06,1.06,1.06,
     1               1.06,1.06,1.06,1.06,1.06,
     1               1.06,1.06,1.06,1.06,1.06,
     1               1.06,1.06,1.06,1.06,1.06,
     1               1.06,1.06,1.06,1.06/

      data c3 /      3.45,3.45,3.45,3.45,3.45,
     1               3.45,3.45,3.45,3.45,3.45,
     1               3.45,3.45,3.45,3.45,3.45,
     1               3.45,3.45,3.45,3.45,3.45,
     1               3.45,3.45,3.45,3.45/

      data cn /      2.996,1.648,2.996,3.292,3.514,
     1               3.563,3.547,3.448,3.312,3.044,
     1               2.831,2.658,2.505,2.261,2.087,
     1               1.812,1.648,1.511,1.470,1.456,
     1               1.465,1.478,1.498,1.502/

      data cM /      4.1840,4.2979,4.1840,4.1879,4.1556,
     1               4.1226,4.1011,4.0860,4.1030,4.1717,
     1               4.2476,4.3184,4.3844,4.4979,4.5881,
     1               4.7571,4.8820,5.0697,5.2173,5.4385,
     1               5.5977,5.7276,5.9891,6.1930/

      data c4 /      -2.1,-2.1,-2.1,-2.1,-2.1,
     1               -2.1,-2.1,-2.1,-2.1,-2.1,
     1               -2.1,-2.1,-2.1,-2.1,-2.1,
     1               -2.1,-2.1,-2.1,-2.1,-2.1,
     1               -2.1,-2.1,-2.1,-2.1/

      data c4a/      -0.5,-0.5,-0.5,-0.5,-0.5,
     1               -0.5,-0.5,-0.5,-0.5,-0.5,
     1               -0.5,-0.5,-0.5,-0.5,-0.5,
     1               -0.5,-0.5,-0.5,-0.5,-0.5,
     1               -0.5,-0.5,-0.5,-0.5/

      data cRB/      50,50,50,50,50,
     1               50,50,50,50,50,
     1               50,50,50,50,50,
     1               50,50,50,50,50,
     1               50,50,50,50/

      data c5/       6.1600,5.1700,6.1600,6.1580,6.1550,
     1               6.1508,6.1441,6.1200,6.0850,5.9871,
     1               5.8699,5.7547,5.6527,5.4997,5.4029,
     1               5.2900,5.2480,5.2194,5.2099,5.2040,
     1               5.2020,5.2010,5.2000,5.2000/

      data c6/       0.4893,0.4407,0.4893,0.4892,0.4890,
     1               0.4888,0.4884,0.4872,0.4854,0.4808,
     1               0.4755,0.4706,0.4665,0.4607,0.4571,
     1               0.4531,0.4517,0.4507,0.4504,0.4501,
     1               0.4501,0.4500,0.4500,0.4500/

      data cHM/      3,3,3,3,3,
     1               3,3,3,3,3,
     1               3,3,3,3,3,
     1               3,3,3,3,3,
     1               3,3,3,3/

      data c7/       0.0512,0.0207,0.0512,0.0512,0.0511,
     1               0.0508,0.0504,0.0495,0.0489,0.0479,
     1               0.0471,0.0464,0.0458,0.0445,0.0429,
     1               0.0387,0.0350,0.0280,0.0213,0.0106,
     1               0.0041,0.0010,0.0000,0.0000/

      data c9/       0.7900,0.3079,0.7900,0.8129,0.8439,
     1               0.8740,0.8996,0.9442,0.9677,0.9660,
     1               0.9334,0.8946,0.8590,0.8019,0.7578,
     1               0.6788,0.6196,0.5101,0.3917,0.1244,
     1               0.0086,0.0000,0.0000,0.0000/

      data c9a/      1.5005,2.6690,1.5005,1.5028,1.5071,
     1               1.5138,1.5230,1.5597,1.6104,1.7549,
     1               1.9157,2.0709,2.2005,2.3886,2.5000,
     1               2.6224,2.6690,2.6985,2.7085,2.7145,
     1               2.7164,2.7172,2.7177,2.7180/

      data cgamma1/  -0.00804,-0.00275,-0.00804,-0.00811,-0.00839,
     1               -0.00875,-0.00912,-0.00973,-0.00975,-0.00883,
     1               -0.00778,-0.00688,-0.00612,-0.00498,-0.00420,
     1               -0.00308,-0.00246,-0.00180,-0.00147,-0.00117,
     1               -0.00106,-0.00102,-0.00096,-0.00094/

      data cgamma2/  -0.00785,-0.00625,-0.00785,-0.00792,-0.00819,
     1               -0.00855,-0.00891,-0.00950,-0.00952,-0.00862,
     1               -0.00759,-0.00671,-0.00598,-0.00486,-0.00410,
     1               -0.00300,-0.00241,-0.00176,-0.00143,-0.00114,
     1               -0.00104,-0.00099,-0.00094,-0.00091/

      data cgamma3/  4,4,4,4,4,
     1               4,4,4,4,4,
     1               4,4,4,4,4,
     1               4,4,4,4,4,
     1               4,4,4,4/

      data phi1/     -0.4417,-0.7861,-0.4417,-0.4340,-0.4177,
     1               -0.4000,-0.3903,-0.4040,-0.4423,-0.5162,
     1               -0.5697,-0.6109,-0.6444,-0.6931,-0.7246,
     1               -0.7708,-0.7990,-0.8382,-0.8663,-0.9032,
     1               -0.9231,-0.9222,-0.8346,-0.7332/

      data phi2/     -0.1417,-0.0699,-0.1417,-0.1364,-0.1403,
     1               -0.1591,-0.1862,-0.2538,-0.2943,-0.3113,
     1               -0.2927,-0.2662,-0.2405,-0.1975,-0.1633,
     1               -0.1028,-0.0699,-0.0425,-0.0302,-0.0129,
     1               -0.0016,0.0000,0.0000,0.0000/

      data phi3/     -0.007010,-0.008444,-0.007010,-0.007279,-0.007354,
     1               -0.006977,-0.006467,-0.005734,-0.005604,-0.005845,
     1               -0.006141,-0.006439,-0.006704,-0.007125,-0.007435,
     1               -0.008120,-0.008444,-0.007707,-0.004792,-0.001828,
     1               -0.001523,-0.001440,-0.001369,-0.001361/

      data phi4/     0.102151,5.410000,0.102151,0.108360,0.119888,
     1               0.133641,0.148927,0.190596,0.230662,0.266468,
     1               0.255253,0.231541,0.207277,0.165464,0.133828,
     1               0.085153,0.058595,0.031787,0.019716,0.009643,
     1               0.005379,0.003223,0.001134,0.000515/

      data phi5/     0.2289,0.2899,0.2289,0.2289,0.2289,
     1               0.2289,0.2290,0.2292,0.2297,0.2326,
     1               0.2386,0.2497,0.2674,0.3120,0.3610,
     1               0.4353,0.4629,0.4756,0.4785,0.4796,
     1               0.4799,0.4799,0.4800,0.4800/

      data phi6/     0.014996,0.006718,0.014996,0.014996,0.014996,
     1               0.014996,0.014996,0.014996,0.014996,0.014988,
     1               0.014964,0.014881,0.014639,0.013493,0.011133,
     1               0.006739,0.005749,0.005544,0.005521,0.005517,
     1               0.005517,0.005517,0.005517,0.005517/

      data phi7/     580.0,459.0,580.0,580.0,580.0,
     1               579.9,579.9,579.6,579.2,577.2,
     1               573.9,568.5,560.5,540.0,512.9,
     1               441.9,391.8,348.1,332.5,324.1,
     1               321.7,320.9,320.3,320.1/

      data phi8/      0.0700, 0.1138, 0.0700, 0.0699, 0.0701,
     1                0.0702, 0.0701, 0.0686, 0.0646, 0.0494,
     1               -0.0019,-0.0479,-0.0756,-0.0960,-0.0998,
     1               -0.0765,-0.0412, 0.0140, 0.0544, 0.1232,
     1                0.1859, 0.2295, 0.2660, 0.2682/

      data tau1/     0.3437,0.2539,0.3437,0.3471,0.3603,
     1               0.3718,0.3848,0.3878,0.3835,0.3719,
     1               0.3601,0.3522,0.3438,0.3351,0.3353,
     1               0.3429,0.3577,0.3769,0.4023,0.4406,
     1               0.4784,0.5074,0.5328,0.5542/

      data tau2/     0.2637,0.2381,0.2637,0.2671,0.2803,
     1               0.2918,0.3048,0.3129,0.3152,0.3128,
     1               0.3076,0.3047,0.3005,0.2984,0.3036,
     1               0.3205,0.3419,0.3703,0.4023,0.4406,
     1               0.4784,0.5074,0.5328,0.5542/

      data sigma1/   0.4458,0.4496,0.4458,0.4458,0.4535,
     1               0.4589,0.4630,0.4702,0.4747,0.4798,
     1               0.4816,0.4815,0.4801,0.4758,0.4710,
     1               0.4621,0.4581,0.4493,0.4459,0.4433,
     1               0.4424,0.4420,0.4416,0.4414/

      data sigma2/   0.3459,0.3554,0.3459,0.3459,0.3537,
     1               0.3592,0.3635,0.3713,0.3769,0.3847,
     1               0.3902,0.3946,0.3981,0.4036,0.4079,
     1               0.4157,0.4213,0.4213,0.4213,0.4213,
     1               0.4213,0.4213,0.4213,0.4213/

      data sigma3/   0.8000,0.7504,0.8000,0.8000,0.8000,
     1               0.8000,0.8000,0.8000,0.8000,0.8000,
     1               0.8000,0.7999,0.7997,0.7988,0.7966,
     1               0.7792,0.7504,0.7136,0.7035,0.7006,
     1               0.7001,0.7000,0.7000,0.7000/

        end block data CY2008Coefficients
